// SVNVimStart

s = Server.local;
s.waitForBoot {
    var module, msg, modules;

    // Connect to OSC
    thisProcess.openUDPPort(5005);
    n = NetAddr.new("0.0.0.0", 5005);
    o = OSCFunc({ arg msg, time, addr, recvPort; [msg, time, addr, recvPort].postln; }, '/radio', n);

    // Create the synth definition and load it
    module = SynthDef.new(\module, { 
        arg hue, saturation, value, pan, gain, octave;
        var oscillator, noise, filter, panner, mixer, frequency, qfactor, noise_level, osc_level, lagtime;


        // Calculate some parameters
        lagtime = 0.5;
        frequency = Lag.kr(200 + 300*hue, lagtime) * (2**octave);
        qfactor = Lag.kr((1 - saturation)**4, lagtime);
        osc_level = Lag.kr(saturation, lagtime);
        noise_level = Lag.kr(1 - osc_level, lagtime);

        // Generate some sounds
        // noise = WhiteNoise.ar(1);
        noise = Crackle.ar(1.99, 1.0);
        filter = BPF.ar(noise, frequency, qfactor) * noise_level;
        //oscillator = SinOsc.ar(frequency) * osc_level;
        oscillator = DPW3Tri.ar(frequency) * osc_level;
        mixer = Mix.ar([filter, oscillator]);
        panner = LinPan2.ar(mixer, pan, 10*HPF.kr(value, 10**(1-octave*2))**2);
        Out.ar(0, panner);
    });
    module.load(s);
    s.sync;

    // Create multiple sound generators
    modules = Array.fill(12, 
        {
            arg index; 
            var pan, octave;
            pan = ((index % 4) - 1.5)/1.5;
            octave = (index / 4).floor;
            "% %\n".postf(pan, octave);
            Synth.new(\module, 
                [\hue, 0.5, \saturation, 0.1, \value, 0.5, pan, 0.5, \gain, 0.9, \octave, octave]
            )
        }
    );


    // Hook up OSC
    f = { |msg, time, addr|
        if(msg[0] == '/radio') {
            if(msg[1]<modules.size){
                modules[msg[1]].set(\hue, msg[2]);
                modules[msg[1]].set(\saturation, msg[3]);
                modules[msg[1]].set(\value, msg[4]);
                modules[msg[1]].set(\pan, msg[5]);
            }
        }
    };
    thisProcess.addOSCRecvFunc(f);
};
